% Anne Collins
% Brown University, Providence, RI, USA
% Laboratory for Neural Computation and Cognition
% Michael Frank.

% November 2013:
% Key simulations of the OpAL model
% Submitted to Psych Review


% Different cells provide simulations of the probabilistic selection task,
% effects of asymmetric learning rates and beta parameters on choice bias,
% interaction between parameters simulations, comparison with classic RL
% gain loss model, and Effort (lever press task) simulations.
% the code can be run as is, so long as OpAL.m and RLGainLoss.m functions
% are accessible to the path.


%% 1. PST OpAL: chA-avB as a function of asymmetry in [beta,alpha]
clear all

% set the baseline parameters
para = [.1 .1 .1];% alphaG, alphaN, alphaC

% set the probability of reward investigated
seuils = [.5:.1:1];
% set the range of beta values investigated
betas = [0:.2:2];
% set initia values 
Init = [1 1 1 1 .5 .5];%[G(B), G(A), N(B), N(A), Q(B),Q(A)].
% set reward value
val = 1;
% force uniform sampling in learning phase
sample = .5;

% lenght of learning
duration = 100;

% run OpAL for different probabilities of reward, store final model values
os=[];
for si = seuils
    % output is [V(B),V(A), G(B),G(A), N(B),N(A)]
    ou = OpAL(para,Init,si,val,duration,sample);
   os = [os;ou]; 
end

% run OpAL specifically when p(r|A) = p(R|B) = .5 (case M in paper)
ou5 = OpAL(para,Init,.5,val,duration,sample);
% store final act5GN = [G(M), N(M)]. (averaged over A, B, since same
% value).
act5GN = .5*[ou5(3)+ou5(4),ou5(5)+ou5(6)];

figure
subplot(2,1,1)
t = 0;
for be = betas
    t = t+1;
    % build Act values for A, B: Act = betaG*G - betaN*N.
    % here, betaG = be; betaN = 2-be.
    % G is 3:4 in os, N is 5:6 in os.
    os(:,7:8) = be*os(:,3:4)-(2-be)*os(:,5:6);
    % get Act(M)
    act5 = be*act5GN(1)-(2-be)*act5GN(2);
    % bias is (Act(A)-Act(M)) - (Act(M)-Act(B))
    os(:,9) = os(:,8)+os(:,7)-2*act5;

    % color code
    x = (t-1)/(length(betas)-1);
    co = x*[0 1 0] + (1-x)*[1 0 0];
    
    % plot Bias as a function of p(r)
    toplot = os(:,9)';
    hold on
    plot(seuils,toplot,'b','Marker','+','MarkerEdgeColor',co,'linewidth',2,'MarkerSize',2) 
    axis([.5 1 -1 1])  
end
ylabel('ChA vs. AvB bias')
xlabel('p(R|A) = 1-p(R|B)')
title('Effect of \beta_G vs. \beta_N asymmetry')




% same for learning rate asymmetry

% set the baseline parameters
para = [.1 .1 .1];% alphaG, alphaN, alphaC

% set the probability of reward investigated
seuils = [.5:.1:1];
% set the range of alpha values investigated
alphaGs = [0:.02:.2];
% set initia values 
Init = [1 1 1 1 .5 .5];%[G(B), G(A), N(B), N(A), Q(B),Q(A)].
% set reward value
val = 1;
% force uniform sampling in learning phase
sample = .5;
% set beta weights.
be = 1;

% lenght of learning
duration = 100;

% run OpAL for different probabilities of reward, store final model values
subplot(2,1,2)
t = 0;
for alphaG = alphaGs
    t = t+1;
    
    % change alphaG and alphaN
    para(1)=alphaG;
    para(2) = .2-alphaG;
    
    % run for those params, various probas of reward, save final model
    % values
    os=[];
    for si = seuils
        ou = OpAL(para,Init,si,val,duration,sample);
       os = [os;ou]; 
    end
    % run for M (p(R) = 0.5)
    ou5 = OpAL(para,Init,.5,val,duration,sample);
    act5GN = .5*[ou5(3)+ou5(4),ou5(5)+ou5(6)];

    % Get Act values for A and B
    os(:,7:8) = be*os(:,3:4)-(2-be)*os(:,5:6);
    % get Act value for M
    act5 = be*act5GN(1)-(2-be)*act5GN(2);
    % get bias
    os(:,9) = os(:,8)+os(:,7)-2*act5;

    % color code
    x = (t-1)/(length(alphaGs)-1);
    co = x*[0 1 0] + (1-x)*[1 0 0];

    % plot bias for different asymmetries of learning rates, reward probabilities 
    toplot = os(:,9)';
    hold on
    plot(seuils,toplot,'b','Marker','+','MarkerEdgeColor',co,'linewidth',2,'MarkerSize',2) 
    %axis([.5 1 -1 1])   
end
ylabel('ChA vs. AvB bias')
xlabel('p(R|A) = 1-p(R|B)')

title('Effect of \alpha_G vs. \alpha_N asymmetry')



%% 2. PST chA-avB [beta,alpha]: test of gain loss RL model
clear all

para = [.1 .1];

seuils = [.5:.1:1];
alphaGs = [0:.02:.2];
% initialize values. Only 5:6 matter: V(A), V(B)
Init = [.5 .5 .5 .5 .5 .5];
val = 1;
sample = .5;
duration = 100;

figure
t = 0;
for alphaG = alphaGs
    t = t+1;
    
    % set alphaGain
    para(1)=alphaG;
    % set alphaLoss
    para(2) = .2-alphaG;
    
    % compute output V(A), V(B)
    os=[];
    for si = seuils
        ou = RLGainLoss(para,Init,si,val,duration,sample);
       os = [os;ou]; 
    end
    % compute neutral output V(M)
    ou5 = RLGainLoss(para,Init,.5,val,duration,sample);

    %compute bias
    os(:,3) = os(:,2)+os(:,1)-ou5(1)-ou5(2);

    % color code
    x = (t-1)/(length(alphaGs)-1);
    co = x*[0 1 0] + (1-x)*[1 0 0];

    toplot = os(:,3)';
    % plot
    hold on
    plot(seuils,toplot,'b','Marker','+','MarkerEdgeColor',co,'linewidth',2,'MarkerSize',2) 
    %axis([.5 1 -1 1])   
end
title('RLGainLoss biases for \alpha_G vs. \alpha_N asymmetry')
ylabel('ChA vs. AvB bias')
xlabel('p(R|A) = 1-p(R|B)')


%% 3. Interaction of alpha, beta asymmetry in PST for chA-avB bias
clear all

% set initial parameters
para = [.1 .1 .1];

% probability of reward = .8/.2
si = .8;
% range of beta values
betas = [0:.25:2];
% range of alpha values
alphaGs = [0:.025:.2];
% initial model values [G(A),G(B),N(A),N(B),Q(A),Q(B)]
Init = [1 1 1 1 .5 .5];
% reward value
val = 1;
% force random sample
sample = .5;
% learning duration
duration = 100;

t = 0;
for alphaG = alphaGs
    t = t+1;
    % set learning rates
    para(1)=alphaG;
    para(2) = .2-alphaG;
    
    os=[];
    % get model values for si = .8 (A/B), and M (p(r)=.5).
    ou = OpAL(para,Init,si,val,duration,sample);
    ou5 = OpAL(para,Init,.5,val,duration,sample);
    % for A: G, N values
    actAGN = [ou(3), ou(5)];
    % for B: G, N values
    actBGN = [ou(4), ou(6)];
    % for M: G, N values
    act5GN = .5*[ou5(3)+ou5(4),ou5(5)+ou5(6)];
        
    % compute act values for each betaG/betaN asymmetry
    for be = betas
        %betaG = be; betaN = 2-be.
       actA = be*actAGN(1)-(2-be)*actAGN(2);
       actB = be*actBGN(1)-(2-be)*actBGN(2);
       act5 = be*act5GN(1)-(2-be)*act5GN(2);
       
       %bias = actA+ actB - 2*act5;
       % compute choose A performance
       chA = 1/(1+exp(-(actB - act5)));
       % compute avoid B performance
       avB = 1/(1+exp(-(act5 - actA)));
       % compute choose A vs avoid B bias
       bias = chA - avB;
       % compute overall performance
       perf = chA+avB;
       % store
       os = [os;[bias chA avB perf]];
       
    end
    bl(t,:,:) = os;
end

% do the plotting
figure
t = 0;
for al = alphaGs
    t = t+1;
    % get values for a given alpha
    os = squeeze(bl(t,:,:));
    
    % color code
    x = (t-1)/(length(alphaGs)-1);
    co = x*[0 1 0] + (1-x)*[1 0 0];
    
    % plot values for all betas, with alphas fixed
    subplot(3,length(alphaGs),t)
    toplot = os(:,1)';
    hold on
    plot(betas,toplot,'Color',co,'linewidth',1,'MarkerSize',2) 
    axis([0 2 -.2 .2])
    title(['\alpha_G =',num2str(al)])
    ylabel('bias')
    xlabel('\beta_G = 2- \beta_N') 
    
    subplot(3,length(alphaGs),length(alphaGs)+t)
    toplot = os(:,2)';
    hold on
    plot(betas,toplot,'Color',co,'linewidth',1,'MarkerSize',2) 
    toplot = os(:,3)';
    plot(betas,toplot,'--','Color',co,'linewidth',1,'MarkerSize',2) 
    axis([0 2 .5 .85])
    title(['\alpha_G =',num2str(al)])
    ylabel('chA/AvB')
    xlabel('\beta_G = 2- \beta_N') 
    
    
    subplot(3,length(alphaGs),2*length(alphaGs)+t)
    toplot = os(:,4)'/2;
    hold on
    plot(betas,toplot,'Color',co,'linewidth',1,'MarkerSize',2) 
    axis([0 2 .5 .75])
    xlabel('\beta_G = 2- \beta_N') 
 
    
end


% do the plotting
figure
t = 0;
for be = betas
    t = t+1;
    % get values for a given betaG, betaN
    os = squeeze(bl(:,t,:));
    
    % color code
    x = (t-1)/(length(betas)-1);
    co = x*[0 1 0] + (1-x)*[1 0 0];
    
    
    % plot values for all alphas, with betas fixed
    subplot(3,length(betas),t)
    toplot = os(:,1)';
    hold on
    plot(alphaGs,toplot,'Color',co,'linewidth',1,'MarkerSize',2) 
    axis([0 .2 -.2 .2])
    title(['\beta_G =',num2str(be)])
    ylabel('bias')
    xlabel('\alpha_G = .2- \alpha_N') 
    
    subplot(3,length(betas),length(betas)+t)
    toplot = os(:,2)';
    hold on
    plot(alphaGs,toplot,'Color',co,'linewidth',1,'MarkerSize',2) 
    toplot = os(:,3)';
    plot(alphaGs,toplot,'--','Color',co,'linewidth',1,'MarkerSize',2) 
    axis([0 .2 .5 .85])
    title(['\beta_G =',num2str(be)])
    ylabel('chA/avB')
    xlabel('\alpha_G = .2- \alpha_N') 
    
    subplot(3,length(betas),2*length(betas)+t)
    toplot = os(:,4)'/2;
    hold on
    plot(alphaGs,toplot,'Color',co,'linewidth',1,'MarkerSize',2) 
    axis([0 .2 .5 .75])
    title(['\beta_G =',num2str(be)])
    ylabel('Perf')
    xlabel('\alpha_G = .2- \alpha_N')  
end


%% 4. Same as previous cell, but for RLGainLoss model
clear all

para = [.1 .1 .1];

seuils = .8;si = seuils;
% change softmax betas
betas = [2 5 10];%
% alphaGain
alphaGs = [0:.025:.2];
Init = [1 1 1 1 .5 .5];
val = 1;
sample = .5;
be = 1;
duration = 100;

% run for various gain loss learning rates, store
t = 0;
for alphaG = alphaGs
    t = t+1;
    
    para(1)=alphaG;
    para(2) = .2-alphaG;
    
    os=[];
    ou = RLGainLoss(para,Init,si,val,duration,sample);
    ou5 = RLGainLoss(para,Init,.5,val,duration,sample);
    actAGN = [ou(1) 0];
    actBGN = [ou(2),0];
    act5GN = .5*[ou5(1)+ou5(2),0];
        
    for be = betas
       actA = be*actAGN(1)-(2-be)*actAGN(2);
       actB = be*actBGN(1)-(2-be)*actBGN(2);
       act5 = be*act5GN(1)-(2-be)*act5GN(2);
       
       bias = actA+ actB - 2*act5;
       chA = 1/(1+exp(-(actB - act5)));
       avB = 1/(1+exp(-(act5 - actA)));
       bias = chA - avB;
       perf = chA+avB;
       os = [os;[bias chA avB perf]];
       
    end
    bl(t,:,:) = os;
end


figure
for be = 5%betas
    t = find(betas==be);
    os = squeeze(bl(:,t,:));
    
    
    x = (t-1)/(length(betas)-1);
    co = x*[0 1 0] + (1-x)*[1 0 0];
    subplot(3,1,1)
    toplot = os(:,1)';
    hold on
    plot(alphaGs,toplot,'Color',co,'linewidth',1,'MarkerSize',2)    
    ylabel('Bias')
    xlabel('\alpha_{Gain} = .2- \alpha_{Loss}')  
    
    
    subplot(3,1,2)
    toplot = os(:,2)';
    hold on
    plot(alphaGs,toplot,'Color',co,'linewidth',1,'MarkerSize',2) 
    toplot = os(:,3)';
    plot(alphaGs,toplot,'--','Color',co,'linewidth',1,'MarkerSize',2)  
    ylabel('ChA/AvB')
    xlabel('\alpha_{Gain} = .2- \alpha_{Loss}')  
    
    
    subplot(3,1,3)
    toplot = os(:,4)'/2;
    hold on
    plot(alphaGs,toplot,'Color',co,'linewidth',1,'MarkerSize',2) 
    ylabel('Perf')
    xlabel('\alpha_{Gain} = .2- \alpha_{Loss}')  
end



%% 5. effort, lever press task. Effect of [beta,alpha]

%params for DoreaFig v3: para = [.1 .1 .02], betas =[.5 1], alphaGs =
%[.075, 1], only alphaN modulated, Init = [1 1 1 1 .5 .5].

clear all

%set parameters
para = [.1 .1 .02];% alphaG, alphaN, alphaC
% set probabilities of reward (proba = 1./seuils)
seuils = [2:2:20];
hold off
% set betaN values to investigate (betaN = 2-betas). betaG is fixed = 1;
betas = [.5 1];
% set alphaN values to investigate (alphaN = 2-alphas). alphaG is fixed =
% .1;
alphas = [.075 .1];

% initialize values
Init = [1 1 1 1 .5 .5];
% reward value
val = 1;
% always sample the same (forced choice, single lever)
sample = 1;% force choice of 2nd option
%length of learning
duration = 100;% 2*duration
% color of conditions
couleur = {'k','b'};

t = 0;
count = 0;
figure
for ai = alphas
    t = t+1;
    
    %para(1)= 1*alphaG;
    % set alphaN
    para(2) = 1*(.2-ai);
    
    % run OpAL for learning param
    os=[];
    for si = seuils
        ou = OpAL(para,Init,1/si,val,duration,sample);
        os = [os;ou]; 
    end

    % get choice probas for different betas
    for be = betas
        
        % color code
        x = (find(betas ==be) -1)/(length(betas)-1);
        co = x*[0 1 0] + (1-x)*[1 0 0];
        
        % get Act value
        os(:,7:8) = 1*os(:,3:4)-(2-be)*os(:,5:6);

        % plot Act value of chosen
        toplot = os(:,8)';
        plot(seuils,toplot,couleur{t},'Marker','+','MarkerEdgeColor',co,'linewidth',2,'MarkerSize',2)
        hold on
        count = count+1;
        legendName{count} = ['\alpha_N = ', num2str(para(2)), ...
            '\alpha_G = ', num2str(para(1)),...
            '\beta_N = ', num2str(2-be), ...
            '\beta_G = ', num2str(be)];
    end 

end
%axis([.5 1 -1 .5]) 
xlabel('number of "lever presses" needed for r = 1 (1/p(r))')  
ylabel('Act value')
title(['Effort willingness for reward'])
axis
legend(legendName)
